@@ -50,7 +50,7 @@ namespace oslogin_utils {
 
 // Metadata server URL.
 static const char kMetadataServerUrl[] =
-    "http://metadata.google.internal/computeMetadata/v1/oslogin/";
+    "http://169.254.169.254/computeMetadata/v1/oslogin/";
 
 // BufferManager encapsulates and manages a buffer and length. This class is not
 // thread safe.
@@ -121,7 +121,7 @@ class NssCache {
   // response is expected to be a JSON array of passwd or group entries. Returns
   // true on success.
   bool LoadJsonUsersToCache(string response);
-  bool LoadJsonGroupsToCache(string response);
+  bool LoadJsonGroupsToCache(string response, int* errnop);
 
   // Helper method for get(pw|gr)ent nss methods. Each call will iterate through the
   // OsLogin database and return the next entry.  Internally, the cache will
@@ -247,6 +247,7 @@ bool ParseJsonToUsers(const string& json, std::vector<string> *users);
 // ssh_keys. A key is considered valid if it's expiration date is greater than
 // current unix time.
 std::vector<string> ParseJsonToSshKeys(const string& json);
+std::vector<string> ParseJsonToSshKeysSk(const string& json);
 
 // Parses a JSON object and returns the value associated with a given key.
 bool ParseJsonToKey(const string& json, const string& key, string* response);
