@@ -177,8 +177,9 @@ cleanup:
   return ret;
 }
 
-bool NssCache::LoadJsonGroupsToCache(string response) {
+bool NssCache::LoadJsonGroupsToCache(string response, int* errnop) {
   Reset();
+  *errnop = ENOENT;
 
   json_object* root = NULL;
   root = json_tokener_parse(response.c_str());
@@ -198,12 +199,14 @@ bool NssCache::LoadJsonGroupsToCache(string response) {
     goto cleanup;
   }
   // A page_token of 0 for groups is different than for users. This is the last
-  // page, but it WILL contain groups.
+  // page, but it WILL contain groups if there are any.
   if (page_token_ == "0") {
     on_last_page_ = true;
     page_token_ = "";
   }
   if (!json_object_object_get_ex(root, "posixGroups", &groups)) {
+    // Valid JSON but no groups, set ENOMSG as a 'no groups' code.
+    *errnop = ENOMSG;
     goto cleanup;
   }
   if (json_object_get_type(groups) != json_type_array) {
@@ -218,6 +221,7 @@ bool NssCache::LoadJsonGroupsToCache(string response) {
     entry_cache_.push_back(json_object_to_json_string_ext(group, JSON_C_TO_STRING_PLAIN));
   }
   ret = true;
+  *errnop = 0;
 
 cleanup:
   json_object_put(root);
@@ -285,11 +289,15 @@ bool NssCache::NssGetgrentHelper(BufferManager* buf, struct group* result, int*
       *errnop = ENOMSG;
       return false;
     }
-    // General failure to load the cache occurred.
-    if (!status || http_code != 200 || response.empty() || !LoadJsonGroupsToCache(response)) {
+    // Failed to make the request or empty response.
+    if (!status || http_code != 200 || response.empty()) {
       *errnop = ENOENT;
       return false;
     }
+    // General failure to load the cache occurred.
+    if (!LoadJsonGroupsToCache(response, errnop)) {
+      return false;
+    }
   }
 
   if (!HasNextEntry() || !GetNextGroup(buf, result, errnop)) {
@@ -542,7 +550,7 @@ bool ParseJsonToGroup(const string& json, struct group* result, BufferManager*
   result->gr_gid = gr_gid;
   if (!buf->AppendString("", &result->gr_passwd, errnop))
     goto cleanup;
-  if (!buf->AppendString((char*)json_object_get_string(name), &result->gr_name,
+  if (!buf->AppendString(json_object_get_string(name), &result->gr_name,
                          errnop))
     goto cleanup;
 
@@ -599,7 +607,7 @@ std::vector<string> ParseJsonToSshKeys(const string& json) {
         if (val_type != json_type_string) {
           continue;
         }
-        key_to_add = (char*)json_object_get_string(val);
+        key_to_add = json_object_get_string(val);
       }
       if (string_key == "expirationTimeUsec") {
         if (val_type == json_type_int || val_type == json_type_string) {
@@ -624,6 +632,62 @@ cleanup:
   return result;
 }
 
+std::vector<string> ParseJsonToSshKeysSk(const string& json) {
+  std::vector<string> result;
+  json_object* security_keys = NULL;
+
+  json_object* root = NULL;
+  root = json_tokener_parse(json.c_str());
+  if (root == NULL) {
+    return result;
+  }
+
+  // Locate the securityKeys array.
+  json_object* login_profiles = NULL;
+  if (!json_object_object_get_ex(root, "loginProfiles", &login_profiles)) {
+    goto cleanup;
+  }
+  if (json_object_get_type(login_profiles) != json_type_array) {
+    goto cleanup;
+  }
+
+  login_profiles = json_object_array_get_idx(login_profiles, 0);
+
+  if (!json_object_object_get_ex(login_profiles, "securityKeys", &security_keys)) {
+    goto cleanup;
+  }
+  if (json_object_get_type(security_keys) != json_type_array) {
+    goto cleanup;
+  }
+
+  {
+    size_t number_of_keys = 0;
+    size_t idx;
+    json_object* security_key = NULL;
+    json_object* public_key = NULL;
+    string key_to_add = "";
+
+    number_of_keys = json_object_array_length(security_keys);
+    for (idx = 0; idx < number_of_keys; idx++) {
+      security_key = json_object_array_get_idx(security_keys, idx);
+      if (json_object_get_type(security_key) != json_type_object) {
+        goto cleanup;
+      }
+      if (!json_object_object_get_ex(security_key, "publicKey", &public_key)) {
+        goto cleanup;
+      }
+
+      key_to_add = json_object_get_string(public_key);
+      result.push_back(key_to_add);
+      key_to_add.clear();
+    }
+  }
+
+cleanup:
+  json_object_put(root);
+  return result;
+}
+
 bool ParseJsonToPasswd(const string& json, struct passwd* result, BufferManager*
                        buf, int* errnop) {
   *errnop = EINVAL;
@@ -699,7 +763,7 @@ bool ParseJsonToPasswd(const string& json, struct passwd* result, BufferManager*
       if (val_type != json_type_string) {
         goto cleanup;
       }
-      if (!buf->AppendString((char*)json_object_get_string(val),
+      if (!buf->AppendString(json_object_get_string(val),
                              &result->pw_name, errnop)) {
         goto cleanup;
       }
@@ -707,7 +771,7 @@ bool ParseJsonToPasswd(const string& json, struct passwd* result, BufferManager*
       if (val_type != json_type_string) {
         goto cleanup;
       }
-      if (!buf->AppendString((char*)json_object_get_string(val),
+      if (!buf->AppendString(json_object_get_string(val),
                              &result->pw_dir, errnop)) {
         goto cleanup;
       }
@@ -715,7 +779,7 @@ bool ParseJsonToPasswd(const string& json, struct passwd* result, BufferManager*
       if (val_type != json_type_string) {
         goto cleanup;
       }
-      if (!buf->AppendString((char*)json_object_get_string(val),
+      if (!buf->AppendString(json_object_get_string(val),
                              &result->pw_shell, errnop)) {
         goto cleanup;
       }
@@ -815,12 +879,10 @@ bool ParseJsonToKey(const string& json, const string& key, string* response) {
 
 
   if (!json_object_object_get_ex(root, key.c_str(), &json_response)) {
-    json_object_put(root);
     goto cleanup;
   }
 
   if (!(c_response = json_object_get_string(json_response))) {
-    json_object_put(root);
     goto cleanup;
   }
 
